@@ -219,6 +219,41 @@ cli_opts = [
                     'the bare metal introspection service when the '
                     '``ironic-collect-introspection-data`` program is '
                     'executing in daemon mode.'),
+    cfg.StrOpt('ntp_server',
+               default=APARAMS.get('ipa-ntp-server', None),
+               help='Address of a single NTP server against which the '
+                    'agent should sync the hardware clock prior to '
+                    'rebooting to an instance.'),
+    cfg.BoolOpt('fail_if_clock_not_set',
+                default=False,
+                help='If operations should fail if the clock time sync '
+                     'fails to complete successfully.'),
+    cfg.StrOpt('agent_token',
+               default=APARAMS.get('ipa-agent-token'),
+               help='Pre-shared token to use when working with the '
+                    'ironic API. This value is typically supplied by '
+                    'ironic automatically.'),
+    cfg.BoolOpt('agent_token_required',
+                default=APARAMS.get('ipa-agent-token-required', False),
+                help='Control to enforce if API command requests should '
+                     'enforce token validation. The configuration provided '
+                     'by the conductor MAY override this and force this '
+                     'setting to be changed to True in memory.'),
+    cfg.IntOpt('image_download_connection_timeout', min=1,
+               default=APARAMS.get(
+                   'ipa-image-download-connection-timeout', 60),
+               help='The connection timeout (in seconds) when downloading '
+                    'an image. Does not affect the whole download.'),
+    cfg.IntOpt('image_download_connection_retries', min=0,
+               default=APARAMS.get('ipa-image-download-connection-retries', 2),
+               help='How many times to retry the connection when downloading '
+                    'an image. Also retries on failure HTTP statuses.'),
+    cfg.IntOpt('image_download_connection_retry_interval', min=0,
+               default=APARAMS.get(
+                   'ipa-image-download-connection-retry-interval', 5),
+               help='Interval (in seconds) between two attempts to establish '
+                    'connection when downloading an image.'),
+
 ]
 
 CONF.register_cli_opts(cli_opts)
