@@ -70,12 +70,28 @@ def _download_with_proxy(image_info, url, image_id):
         os.environ['no_proxy'] = no_proxy
     proxies = image_info.get('proxies', {})
     verify, cert = utils.get_ssl_client_options(CONF)
-    resp = requests.get(url, stream=True, proxies=proxies,
-                        verify=verify, cert=cert)
-    if resp.status_code != 200:
-        msg = ('Received status code {} from {}, expected 200. Response '
-               'body: {}').format(resp.status_code, url, resp.text)
-        raise errors.ImageDownloadError(image_id, msg)
+    resp = None
+    for attempt in range(CONF.image_download_connection_retries + 1):
+        try:
+            resp = requests.get(url, stream=True, proxies=proxies,
+                                verify=verify, cert=cert,
+                                timeout=CONF.image_download_connection_timeout)
+            if resp.status_code != 200:
+                msg = ('Received status code {} from {}, expected 200. '
+                       'Response body: {}').format(resp.status_code, url,
+                                                   resp.text)
+                raise errors.ImageDownloadError(image_id, msg)
+        except (errors.ImageDownloadError, requests.RequestException) as e:
+            if (attempt == CONF.image_download_connection_retries
+                    # NOTE(dtantsur): do not retry 4xx status codes
+                    or (resp and resp.status_code < 500)):
+                raise
+            else:
+                LOG.warning('Unable to connect to %s, retrying. Error: %s',
+                            url, e)
+                time.sleep(CONF.image_download_connection_retry_interval)
+        else:
+            break
     return resp
 
 
@@ -196,6 +212,11 @@ def _write_image(image_info, device):
     totaltime = time.time() - starttime
     LOG.info('Image {} written to device {} in {} seconds'.format(
              image, device, totaltime))
+    try:
+        disk_utils.fix_gpt_partition(device, node_uuid=None)
+    except exception.InstanceDeployFailure:
+        # Note: the catch internal to the helper method logs any errors.
+        pass
     return uuids
 
 
@@ -269,7 +290,16 @@ class ImageDownload(object):
             self._hash_algo = hashlib.new(algo)
             self._expected_hash_value = image_info.get('os_hash_value')
         elif image_info.get('checksum'):
-            self._hash_algo = hashlib.md5()
+            try:
+                self._hash_algo = hashlib.md5()
+            except ValueError as e:
+                message = ('Unable to proceed with image {} as the legacy '
+                           'checksum indicator has been used, which makes use '
+                           'the MD5 algorithm. This algorithm failed to load '
+                           'due to the underlying operating system. Error: '
+                           '{}').format(image_info['id'], str(e))
+                LOG.error(message)
+                raise errors.RESTError(details=message)
             self._expected_hash_value = image_info['checksum']
         else:
             message = ('Unable to verify image {} with available checksums. '
@@ -391,7 +421,8 @@ def _validate_image_info(ext, image_info=None, **kwargs):
         raise errors.InvalidCommandParamsError(
             'Image \'urls\' must be a list with at least one element.')
 
-    if 'checksum' in image_info:
+    checksum = image_info.get('checksum')
+    if checksum is not None:
         if (not isinstance(image_info['checksum'], str)
                 or not image_info['checksum']):
             raise errors.InvalidCommandParamsError(
@@ -401,12 +432,12 @@ def _validate_image_info(ext, image_info=None, **kwargs):
     os_hash_algo = image_info.get('os_hash_algo')
     os_hash_value = image_info.get('os_hash_value')
     if os_hash_algo or os_hash_value:
-        if (not isinstance(os_hash_algo, str) or
-                not os_hash_algo):
+        if (not isinstance(os_hash_algo, str)
+                or not os_hash_algo):
             raise errors.InvalidCommandParamsError(
                 'Image \'os_hash_algo\' must be a non-empty string.')
-        if (not isinstance(os_hash_value, str) or
-                not os_hash_value):
+        if (not isinstance(os_hash_value, str)
+                or not os_hash_value):
             raise errors.InvalidCommandParamsError(
                 'Image \'os_hash_value\' must be a non-empty string.')
         os_hash_checksum_avail = True
@@ -508,6 +539,12 @@ class StandbyExtension(base.BaseAgentExtension):
                  "seconds".format(device, totaltime))
         # Verify if the checksum of the streamed image is correct
         image_download.verify_image(device)
+        # Fix any gpt partition
+        try:
+            disk_utils.fix_gpt_partition(device, node_uuid=None)
+        except exception.InstanceDeployFailure:
+            # Note: the catch internal to the helper method logs any errors.
+            pass
 
     @base.async_command('cache_image', _validate_image_info)
     def cache_image(self, image_info=None, force=False):
@@ -617,6 +654,11 @@ class StandbyExtension(base.BaseAgentExtension):
         :raises: SystemRebootError if the command errors out with an
             unsuccessful exit code.
         """
+        # TODO(TheJulia): When we have deploy/clean steps, we should remove
+        # this upon shutdown. The clock sync deploy step can run before
+        # completing other operations.
+        self._sync_clock(ignore_errors=True)
+
         if command not in ('reboot', 'poweroff'):
             msg = (('Expected the command "poweroff" or "reboot" '
                     'but received "%s".') % command)
@@ -663,3 +705,28 @@ class StandbyExtension(base.BaseAgentExtension):
             error_msg = 'Flushing file system buffers failed. Error: %s' % e
             LOG.error(error_msg)
             raise errors.CommandExecutionError(error_msg)
+
+    # TODO(TheJulia): Once we have deploy/clean steps, this should
+    # become a step, which we ideally have enabled by default.
+    def _sync_clock(self, ignore_errors=False):
+        """Sync the clock to a configured NTP server.
+
+        :param ignore_errors: Boolean option to indicate if the
+                              errors should be fatal. This option
+                              does not override the fail_if_clock_not_set
+                              configuration option.
+        :raises: ClockSyncError if a failure is encountered and
+                 errors are not ignored.
+        """
+        try:
+            utils.sync_clock(ignore_errors=ignore_errors)
+            # Sync the system hardware clock from the software clock,
+            # as they are independent and the HW clock can still drift
+            # with long running ramdisks.
+            utils.execute('hwclock', '-v', '--systohc')
+        except (processutils.ProcessExecutionError,
+                errors.CommandExecutionError) as e:
+            msg = 'Failed to sync hardware clock: %s' % e
+            LOG.error(msg)
+            if CONF.fail_if_clock_not_set or not ignore_errors:
+                raise errors.ClockSyncError(msg)
