@@ -12,7 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import mock
+from unittest import mock
+
 from oslo_concurrency import processutils
 
 from ironic_python_agent import dmi_inspector
